ECGDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ECG.h"
#include "ECGDlg.h"
#include <math.h>

#include "TabMain.h"
#include "TabFourier.h"
#include "TabStat.h"
#include "TabPulse.h"
#include "TabTempo.h"
#include "TabSynchro.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

LPWSTR ECGData::g_sndRes = NULL;

UINT MyThreadFunc(LPVOID lParam);
BOOL PlayResource(LPCWSTR lpName);
// CECGDlg dialog

static UINT BASED_CODE s_StatusIndicators[] =
{
    ID_INDICATOR_STATUS,
    ID_INDICATOR_MESSAGE
};

CECGDlg::CECGDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CECGDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CECGDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CUSTOM1, m_wndMainGraph);
	DDX_Control(pDX, IDC_TAB1, m_Tab);
}

BEGIN_MESSAGE_MAP(CECGDlg, CDialog)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CECGDlg message handlers

BOOL CECGDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_wndBar.Create(this); //We create the status bar
	m_wndBar.SetIndicators( s_StatusIndicators, 2 ); //Set the number of panes 
	
	CRect rect;
	GetClientRect(&rect);
	m_wndBar.SetPaneInfo(0, ID_INDICATOR_STATUS, SBPS_NORMAL, 200/*rect.Width()-100*/ );
	m_wndBar.SetPaneInfo(1, ID_INDICATOR_MESSAGE, SBPS_STRETCH, 0);

	RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, ID_INDICATOR_MESSAGE);

	m_wndMainGraph.Setup( rect.Width()*4, 0, -2048, GetSysColor(15), 0xff );

	m_Tab.AddTab( new CTabMain(this) );
	m_Tab.AddTab( new CTabFourier(this) );
	m_Tab.AddTab( new CTabStat(this) );
	m_Tab.AddTab( new CTabPulse(this) );
	m_Tab.AddTab( new CTabTempo(this) );
	m_Tab.AddTab( new CTabSynchro(this) );
    m_Tab.InitDialogs();
    m_Tab.ActivateTabDialogs();

	SetTimer( 0, 20, NULL );
	SetTimer( 1, 1000, NULL );

	//CWinThread *pThread = AfxBeginThread( MyThreadFunc, this,
	//	THREAD_PRIORITY_NORMAL, 0, 0 );
	return TRUE;  
}


void CECGDlg::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
	CString strRcv;

	int nRecv = 0;
	int nStreaming = 0;
	int i;

	if ( nIDEvent == 1 )
	{
		UpdateStatistics();
		return;
	}

	if ( !m_comm.IsValid() )
	{
		CString strStatus;
		strStatus.Format( _T("Port closed") );
		m_wndBar.SetPaneText(0, strStatus);
		return;
	}

	if ( m_comm.Receive(strRcv) )
	{
		m_ECGData.m_dwBufferReceivedTime = GetTickCount();
		m_ECGData.ReceiveHandler( strRcv );

		nRecv = strRcv.GetLength();
		m_ECGData.m_nBufferReceivedSize = nRecv>>1;

		for (i=0; i<nRecv-1; i+=2)
		{
			unsigned char v0 = (unsigned char)strRcv.GetAt(i);
			unsigned char v1 = (unsigned char)strRcv.GetAt(i+1);
			unsigned char nSyncFrame = v0 & 0x80 ? 1 : 0;
			unsigned char nDataFrame = v0 & 0x40 ? 1 : 0;

			m_ECGData.StreamHandler( nSyncFrame, nDataFrame );

			v0 &= 0x3f;
			unsigned int nVal = 2048-(v0*256+v1);
			AddValue( nVal );
			m_ECGData.m_nBufferReceivedIndex = i>>1;
			m_Tab.ValueHandler( nVal );
		}
		m_Tab.ReceiveHandler();

		CRect rcGraph;
		m_wndMainGraph.GetClientRect( rcGraph );
		m_wndMainGraph.InvalidateRect( rcGraph, FALSE );
	}

	if ( !m_ECGData.m_strNewStream.IsEmpty() )
	{
		m_wndBar.SetPaneText(1, m_ECGData.m_strNewStream);
		m_ECGData.m_strNewStream.Empty();
	}

	m_ECGData.DoStatistics();

	CString strStatus;
	if ( nStreaming )
		strStatus.Format( _T("%03d bytes read, %3.2f spl/s, stream"), nRecv, m_ECGData.m_dbSamples );
	else
		strStatus.Format( _T("%03d bytes read, %3.2f spl/s"), nRecv, m_ECGData.m_dbSamples );
	m_wndBar.SetPaneText(0, strStatus);
	CDialog::OnTimer(nIDEvent);
}

void CECGDlg::UpdateStatistics(void)
{
	int i;
	int nSum = 0;
	int nMin, nMax;
	int nMM1, nMM2, nMM3, nMM4;

	nMin = nMax = m_wndMainGraph.m_arrData[0];
	for ( i=0; i<m_wndMainGraph.m_nWidth; i++ )
	{
		int nSam = m_wndMainGraph.m_arrData[i];
		nSum += nSam;
		if ( nSam < nMin ) nMin = nSam;
		if ( nSam > nMax ) nMax = nSam;
	}

	nMM1 = nMin + (nMax-nMin)/5;
	nMM2 = nMin + (nMax-nMin)*2/5;
	nMM3 = nMin + (nMax-nMin)*3/5;
	nMM4 = nMin + (nMax-nMin)*4/5;

	m_ECGData.m_nMin = nMin;
	m_ECGData.m_nMax = nMax;
	m_ECGData.m_nAvg = nSum / m_wndMainGraph.m_nWidth;

	m_ECGData.m_nThresh = nMax-(nMax-nMin)/10;	// -10%
	
	m_ECGData.m_fRMS = 0;
	for ( i=0; i<m_wndMainGraph.m_nWidth; i++ )
	{
		int nSam = m_wndMainGraph.m_arrData[i];
		if (nSam <= nMM1) m_ECGData.m_fHistogram[0] ++; else
		if (nSam <= nMM2) m_ECGData.m_fHistogram[1] ++; else
		if (nSam <= nMM3) m_ECGData.m_fHistogram[2] ++; else
		if (nSam <= nMM4) m_ECGData.m_fHistogram[3] ++; else
		if (nSam <= nMax) m_ECGData.m_fHistogram[4] ++;
		nSam -= m_ECGData.m_nAvg;
		m_ECGData.m_fRMS += (FLOAT)nSam*(FLOAT)nSam;
	}
	m_ECGData.m_fRMS = sqrt( m_ECGData.m_fRMS/m_wndMainGraph.m_nWidth )/(nMax-nMin);

	m_ECGData.m_fHistogram[0] /= m_wndMainGraph.m_nWidth;
	m_ECGData.m_fHistogram[1] /= m_wndMainGraph.m_nWidth;
	m_ECGData.m_fHistogram[2] /= m_wndMainGraph.m_nWidth;
	m_ECGData.m_fHistogram[3] /= m_wndMainGraph.m_nWidth;
	m_ECGData.m_fHistogram[4] /= m_wndMainGraph.m_nWidth;

	m_ECGData.m_strStat = _T("Invalid");
	if ( m_ECGData.m_fHistogram[3] > m_ECGData.m_fHistogram[0] + m_ECGData.m_fHistogram[1] + 
			m_ECGData.m_fHistogram[2] + m_ECGData.m_fHistogram[4] )
		m_ECGData.m_strStat = _T("Crossed");

	if ( m_ECGData.m_fHistogram[1] > m_ECGData.m_fHistogram[0] + m_ECGData.m_fHistogram[3] + 
		m_ECGData.m_fHistogram[2] + m_ECGData.m_fHistogram[4] )
		m_ECGData.m_strStat = _T("Valid");

//	UpdateData( FALSE );
}

void CECGDlg::AddValue(int nValue)
{
	/*
	static int nLastVal = -1;
	if ( m_ECGData.m_strStat.Compare(_T("Valid")) == 0 )
	{
		// find peaks
		int nThresh = m_ECGData.m_nMax - ((m_ECGData.m_nMax-m_ECGData.m_nMin)>>3);
		
		if ( nLastVal != -1 )
		{
			if ( nLastVal < nThresh && nValue >= nThresh )
			{
				if ( !m_ECGData.m_bThreadLock )
					m_ECGData.m_strThreadPlay = _T("bell");
				// mame
			}
		}

		nLastVal = nValue;
	} else
		nLastVal = -1;
	*/
	m_wndMainGraph.AddValue( nValue );
}

void CECGDlg::ComputeFourier()
{
	static FLOAT farrFourier[2048];
	int i;
	int nWidth = m_wndMainGraph.m_nWidth;
	int nPos = m_wndMainGraph.m_nPos;
	int nAvg = m_ECGData.m_nAvg;

	_ASSERT( nWidth < 2048 );

	for ( i=0; i<nWidth; i++ )
		farrFourier[i] = (m_wndMainGraph.m_arrData[ (nPos-1-i+nWidth)%nWidth]-nAvg)*0.00048828125f;
	for ( ; i<2048; i++ )
		farrFourier[i] = 0;

	m_fourier.ComplexFFT( farrFourier, nWidth, 2048, 1 ); 
	m_ECGData.m_fFourier = m_fourier.vector;
}


BOOL PlayResource(LPCWSTR lpName)
{
    BOOL bRtn;
    LPWSTR lpRes;
    HANDLE hRes;
    HRSRC hResInfo;
    HINSTANCE Nl=AfxGetInstanceHandle();

    /* Find the WAVE resource. */
    hResInfo= FindResource(Nl,lpName,_T("WAVE"));
    if(hResInfo == NULL)
       return FALSE;
    /* Load the WAVE resource. */

    hRes = LoadResource(Nl,hResInfo);
    if (hRes == NULL)
      return FALSE;

    /* Lock the WAVE resource and play it. */
    lpRes=(LPWSTR)LockResource(hRes);
    if(lpRes==NULL)
      return FALSE;

    bRtn = sndPlaySound(lpRes, SND_MEMORY | SND_SYNC);
    if(bRtn == NULL)
      return FALSE;

    /* Free the WAVE resource and return success or failure. */
    FreeResource(hRes);
    return TRUE;
}

UINT MyThreadFunc(LPVOID lParam)
{
	_ASSERT(0);
	CECGDlg *lpDlg = (CECGDlg *)lParam;

	//return 0;
	lpDlg->m_ECGData.m_bThreadRunning = TRUE;
	lpDlg->m_ECGData.m_strThreadPlay.Empty();
	lpDlg->m_ECGData.m_bThreadLock = FALSE;

// {{{
    BOOL bRtn;
    LPWSTR lpRes;
    HANDLE hRes;
    HRSRC hResInfo;
    HINSTANCE Nl=AfxGetInstanceHandle();

    /* Find the WAVE resource. */
    hResInfo= FindResource(Nl,_T("bell"),_T("WAVE"));
	if (!hResInfo)
		MessageBox(NULL, _T("AssertBox"), _T("bla"), 0);
    if(hResInfo == NULL)
       return FALSE;
    /* Load the WAVE resource. */

    hRes = LoadResource(Nl,hResInfo);
    if (hRes == NULL)
      return FALSE;

    /* Lock the WAVE resource and play it. */
    lpRes=(LPWSTR)LockResource(hRes);
    if(lpRes==NULL)
      return FALSE;

	ECGData::g_sndRes = lpRes;
	OutputDebugString(_T("\n\nmy thread starded\n\n"));

// }}}
	while (lpDlg->m_ECGData.m_bThreadRunning)
	{
		if ( !lpDlg->m_ECGData.m_strThreadPlay.IsEmpty() )
		{
			lpDlg->m_ECGData.m_bThreadLock = TRUE;
			CString strSound = lpDlg->m_ECGData.m_strThreadPlay;
			lpDlg->m_ECGData.m_strThreadPlay.Empty();
			lpDlg->m_ECGData.m_bThreadLock = FALSE;

			//PlayResource( strSound );
		    bRtn = sndPlaySound(lpRes, SND_MEMORY | SND_SYNC);
		}
		Sleep(10);
	}
// {{{
    /* Free the WAVE resource and return success or failure. */
    FreeResource(hRes);
// }}}
	return 0;
}