// ECGDlg.h : header file
//

#pragma once
#include "graph.h"
#include "afxwin.h"
#include "SerialComm.h"
#include "multigraph.h"
#include "tab.h"
#include "Fourier.h"

#pragma comment(lib, "winmm.lib")
#include <mmsystem.h>

#define BWDATA 64

/*
#ifndef G_sndRes
#define G_sndRes
LPWSTR g_sndRes = NULL;
#endif
*/
class ECGData
{
public:
	static LPWSTR g_sndRes;
	HANDLE m_hresSnd;

	ECGData()
	{
		m_nStreamIndex = -1;
		m_nBwIndex = -1;
		ZeroMemory(m_aStreamData, sizeof(m_aStreamData) );
		m_fFourier = NULL;
		m_dbSamples = 0;
		m_nThresh = 2000;

		// resources
		m_hresSnd = NULL;
		LPWSTR lpRes;
		HANDLE hRes;
		HRSRC hResInfo;
		HINSTANCE Nl=AfxGetInstanceHandle();

		/* Find the WAVE resource. */
		hResInfo= FindResource(Nl,_T("bell"),_T("WAVE"));
		if(hResInfo == NULL)
		   return;
		/* Load the WAVE resource. */

		hRes = LoadResource(Nl,hResInfo);
		if (hRes == NULL)
		  return ;
		m_hresSnd = hRes;
		/* Lock the WAVE resource and play it. */
		lpRes=(LPWSTR)LockResource(hRes);
		if(lpRes==NULL)
		  return ;

		ECGData::g_sndRes = lpRes;
	}

	~ECGData()
	{
		// resources
		if ( m_hresSnd )
			FreeResource( m_hresSnd );
	}

	void ReceiveHandler( CString &strRcv )
	{
		static int nRecvFrame = 0;
		static LONG s_lLastTime = 0;
		int j;

		if ( m_nBwIndex == -1 && s_lLastTime == 0 ) 
		{
			s_lLastTime = GetTickCount();
		} else
		if ( m_nBwIndex == -1 ) 
		{
			m_nBwIndex = 0;
			m_arrBwTime[0] = GetTickCount() - s_lLastTime;
			m_arrBwCount[0] = strRcv.GetLength();
			s_lLastTime = GetTickCount();
			for ( j = 1; j < BWDATA; j++ )
			{
				m_arrBwTime[j] = m_arrBwTime[0];
				m_arrBwCount[j] = m_arrBwCount[0];
			}
		} else
		{
			//int nPrev = m_nBwIndex; //(m_nBwIndex+31) % 32;
			m_nBwIndex++;
			m_nBwIndex %= BWDATA;

			m_arrBwTime[m_nBwIndex] = GetTickCount() - s_lLastTime;
			s_lLastTime = GetTickCount();
			m_arrBwCount[m_nBwIndex] = strRcv.GetLength();
		}
	}

	void StreamHandler( char nSyncFrame, char nDataFrame )
	{
		if ( nSyncFrame && !nDataFrame )
			m_nStreamIndex = 0;
		if ( nSyncFrame && nDataFrame )
		{
			if ( m_nStreamIndex > 0 && m_nStreamIndex < 256 )
			{
				m_strNewStream = CString ( m_aStreamData, m_nStreamIndex >> 3 );
			}
			m_nStreamIndex = -1;
		}
		if ( !nSyncFrame && m_nStreamIndex >= 0)
		{
			if ( m_nStreamIndex < 128 )
			{
				if ( nDataFrame )
					m_aStreamData[ m_nStreamIndex >> 3 ] |= 1 << (m_nStreamIndex & 7);
				else
					m_aStreamData[ m_nStreamIndex >> 3 ] &= ~(1 << (m_nStreamIndex & 7));
				m_nStreamIndex++;
			} 
		}
	}

	void DoStatistics()
	{
		double dbBpm = 0;
		LONG dwSumTime = 0;
		LONG dwSumCount = 0;
		int j;

		for ( j = 0; j < BWDATA; j++ )
		{
			dwSumTime += m_arrBwTime[j];
			dwSumCount += m_arrBwCount[j];
		}
		if (dwSumTime == 0)
			dwSumTime = -1;

		dbBpm = (double)dwSumCount / (double)dwSumTime;	// bytes per ms
		dbBpm *= 1000;	// per second
		dbBpm *= 0.5; //bytes to samples
		m_dbSamples = dbBpm;
	}

	int m_nStreamIndex;
	char m_aStreamData[256];
	DWORD m_arrBwTime[BWDATA];
	DWORD m_arrBwCount[BWDATA];
	int	m_nBwIndex;

	FLOAT m_fHistogram[5];
	FLOAT m_fRMS;
	int m_nAvg;
	int m_nMin;
	int m_nMax;
	int m_nThresh;
	FLOAT *m_fFourier;
	CString m_strNewStream;
	DOUBLE m_dbSamples;
	CString m_strStat;
	// thread

	BOOL m_bThreadRunning;
	CString m_strThreadPlay;
	BOOL m_bThreadLock;

	DWORD m_dwBufferReceivedTime;
	int m_nBufferReceivedIndex;
	int m_nBufferReceivedSize;
};

// CECGDlg dialog
class CECGDlg : public CDialog
{
public:
	ECGData		m_ECGData;
	CFourier	m_fourier;

// Construction
public:
	CECGDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_ECG_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	DECLARE_MESSAGE_MAP()

public:
	CGraph m_wndMainGraph;
	CStatusBar m_wndBar;
	CSerialComm	m_comm;

public:
	afx_msg void OnTimer(UINT_PTR nIDEvent);
public:
	void UpdateStatistics(void);
public:
	void AddValue(int nValue);
public:
	CTab m_Tab;
	void ComputeFourier();
	BOOL PlayResource(LPCWSTR lpName);
};
