#pragma once

#include "Graph.h"

struct MultiGraphPoint 
{
	FLOAT x;
	FLOAT y;

public:
	MultiGraphPoint() : x(0), y(0)
	{
	}
	MultiGraphPoint( FLOAT _x, FLOAT _y ) : x(_x), y(_y)
	{
	}
};

class MultiGraphLayer
{
public:
	COLORREF m_clrColor;
	int m_nWidth;
	int m_nStyle;
	BOOL m_bSkip;

	CArray <MultiGraphPoint, MultiGraphPoint> m_arrData;

	MultiGraphLayer( COLORREF clrColor, int nWidth )
	{
		m_clrColor = clrColor;
		m_nWidth = nWidth;
		m_nStyle = PS_SOLID;
		m_bSkip = FALSE;
	}

	void AddPoint( MultiGraphPoint &point )
	{
		m_arrData.Add( point );
	}
};

typedef MultiGraphLayer* LPMultiGraphLayer;

// CMultiGraph

class CMultiGraph : public CWnd
{
	DECLARE_DYNAMIC(CMultiGraph)
	
	CArray <LPMultiGraphLayer, LPMultiGraphLayer> m_arrLayers;
	//CMap <TCHAR, TCHAR, MultiGraphOverlay, MultiGraphOverlay> m_arrOverlay;
	LPMultiGraphLayer m_lpCurrentLayer;
	int m_nLimit;
	COLORREF m_clrBack;

public:
	CMultiGraph();
	virtual ~CMultiGraph();
	BOOL RegisterWindowClass();

	void SkipCurrentLayer( BOOL bSkip )
	{
		if ( m_lpCurrentLayer )
			m_lpCurrentLayer->m_bSkip = bSkip;
	}

	void Setup( COLORREF clrBack, int nLimit )
	{
		m_clrBack = clrBack;
		m_nLimit = nLimit;
	}

	void NewSeries(COLORREF clrColor, int nWidth)
	{
		if ( m_lpCurrentLayer )
			m_lpCurrentLayer->m_bSkip = FALSE;

		m_lpCurrentLayer = new MultiGraphLayer( clrColor, nWidth );
		m_arrLayers.Add( m_lpCurrentLayer );
		if ( m_nLimit > 0 && m_arrLayers.GetSize() >= m_nLimit )
		{
			delete m_arrLayers[0];
			m_arrLayers.RemoveAt( 0 );
		}
	}
	
	void AddPoint( FLOAT x, FLOAT y )
	{
		if ( !m_lpCurrentLayer )
			return;

		_ASSERT( m_lpCurrentLayer );

		if ( x > 1.0 )
			x -= 1.0;

		m_lpCurrentLayer->m_arrData.Add( MultiGraphPoint( x, y ) );
	}

	LPMultiGraphLayer GetCurrentLayer( )
	{
		return m_lpCurrentLayer;
	}

	void Paint( CRect rcRect, CDC *lpDC, int n )
	{
		_ASSERT( lpDC );

		LPMultiGraphLayer lpLayer = m_arrLayers[n];
		_ASSERT( lpLayer );

		COLORREF clrA = lpLayer->m_clrColor;
		COLORREF clrB = m_clrBack;
		
		COLORREF clrLine;
		BOOL bTop = (lpLayer == m_lpCurrentLayer);

		if ( bTop && m_lpCurrentLayer->m_bSkip )
			return;

		if ( m_nLimit <= 0 )
			clrLine = clrA;
		else
		{
			int nAlpha = 0;

			if (!bTop)
				nAlpha = (m_nLimit - n)*255/m_nLimit;

			if (nAlpha > 255)
				nAlpha = 255;
			if (nAlpha < 0)
				nAlpha = 0;

			clrLine = CGraph::ColorMid( clrA, clrB, nAlpha );
		}

		CPen myPen( lpLayer->m_nStyle, lpLayer->m_nWidth, clrLine );
		HPEN hOldPen = (HPEN) lpDC->SelectObject( myPen );
		int i, nCount = (int)lpLayer->m_arrData.GetSize();
		int nWidth = rcRect.Width();
		int nHeight = rcRect.Height();
		int ox = 1000;
		for ( i = 0; i < nCount; i++ )
		{
			MultiGraphPoint *pt = lpLayer->m_arrData.GetData() + i;
			int x = rcRect.left + (int)((FLOAT)nWidth * pt->x);
			int y = rcRect.top + (int)((FLOAT)nHeight * pt->y);
			(ox > x) ?
				lpDC->MoveTo( x, y ) :
				lpDC->LineTo( x, y );
			ox = x;
		}

		lpDC->SelectObject( hOldPen );
	}
	
protected:
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnPaint();
public:
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
};


