// Graph.cpp : implementation file
//

#include "stdafx.h"
#include "ECG.h"
#include "PhaseWheel.h"
#include <math.h>
#include "Graph.h"

#define GRAPH_CLASSNAME    _T("PWheelCtrl")  // Window class name

// CGraph

IMPLEMENT_DYNAMIC(CPWheel, CWnd)

CPWheel::CPWheel()
{
	RegisterWindowClass();
}

CPWheel::~CPWheel()
{
}

BOOL CPWheel::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, GRAPH_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class

        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = GRAPH_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}


BEGIN_MESSAGE_MAP(CPWheel, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()


// CPWheel message handlers

void CPWheel::OnPaint()
{
	CDC *lpDC = &m_dcBuffer;

	CPaintDC dc(this); // device context for painting

	CRect rcRect, rcWndRect;
	GetClientRect( &rcWndRect );

	rcRect = rcWndRect;
	rcRect.right += rcRect.Width();
	rcRect.bottom += rcRect.Height();

	CBrush myBrush( m_clrBack );
	lpDC->FillRect( rcRect, &myBrush );

	CPoint cpCenter( rcWndRect.Width(), rcWndRect.Height() );
	int nRadius = (int) (min( rcWndRect.Width(), rcWndRect.Height() ) * 0.9f);

	CPen penCircle(PS_SOLID, 2, CGraph::ColorMid( m_clrLine, m_clrBack, 128 ) );
	HPEN hOldPen = (HPEN)lpDC->SelectObject( penCircle );
	CBrush brush(m_clrBack);
	HBRUSH hOldBrush = (HBRUSH)lpDC->SelectObject( brush );

	lpDC->Ellipse(cpCenter.x-nRadius, cpCenter.y-nRadius, cpCenter.x+nRadius, cpCenter.y+nRadius);

	CPen penNone(PS_NULL, 0, (COLORREF)0);
	lpDC->SelectObject( penNone );
	for (int i=0; i<m_arrRegions.GetSize(); i++)
	{
		CBrush piebrush( m_arrRegions[i].clrColor );
		lpDC->SelectObject( piebrush );
		lpDC->Pie( m_arrRegions[i].rcRect, m_arrRegions[i].ptMin, m_arrRegions[i].ptMax );
	}

	CPen penLine(PS_SOLID, 3, m_clrLine);
	lpDC->SelectObject( penLine );

	FLOAT fPhase = (m_fPhase-m_fMin) * 3.1415926f * 2.0f / (m_fMax-m_fMin);

	CPoint cpPhasor( (int) (cos( fPhase )*nRadius), (int) (-sin(fPhase) *nRadius) );
	lpDC->MoveTo( cpCenter );
	lpDC->LineTo( cpCenter+cpPhasor );
	lpDC->SelectObject( hOldPen );
	lpDC->SelectObject( hOldBrush );

	if (lpDC != &dc)
	{
		dc.SetStretchBltMode(HALFTONE);
		dc.SetBrushOrg(0, 0);
		dc.StretchBlt(0, 0, rcWndRect.Width(), rcWndRect.Height(), lpDC, 0, 0, rcRect.Width(), rcRect.Height(), SRCCOPY );
	}
}

void CPWheel::AddRegion(FLOAT fMin, FLOAT fMax, COLORREF clrReg)
{
	Region reg;
	reg.fMin = fMin;
	reg.fMax = fMax;
	reg.clrColor = CGraph::ColorMid( clrReg, m_clrBack, 220 );
	

	CRect rcRect;
	GetClientRect(rcRect);
	//rcRect.right += rcRect.Width();
	//rcRect.bottom += rcRect.Height();

	CPoint cpCenter( rcRect.Width(), rcRect.Height() );
	int nRadius = (int) (min( rcRect.Width(), rcRect.Height() ) * 0.9f)-1;

	FLOAT fAngle = (fMin - m_fMin) * 3.141592f * 2.0f / (m_fMax-m_fMin);
	reg.ptMin = CPoint( (int) (cos( fAngle ) * nRadius), -(int) (sin( fAngle ) * nRadius) );
	
	fAngle = (fMax - m_fMin) * 3.141592f * 2.0f / (m_fMax-m_fMin);
	reg.ptMax = CPoint( (int) (cos( fAngle ) * nRadius), -(int) (sin( fAngle ) * nRadius) );

	reg.ptMin += cpCenter;
	reg.ptMax += cpCenter;

	reg.rcRect = CRect(cpCenter.x - nRadius, cpCenter.y - nRadius, cpCenter.x + nRadius, cpCenter.y + nRadius);
	
	m_arrRegions.Add(reg);
}

void CPWheel::Setup(FLOAT fMin, FLOAT fMax, COLORREF clrBack, COLORREF clrLine, int nType)
{
	m_fMin = fMin;
	m_fMax = fMax;
	m_clrLine = clrLine;
	m_clrBack = clrBack;
	m_fPhase = 0;

	CRect rcRect;
	GetClientRect( &rcRect );

	m_nDCWidth = rcRect.Width()*2;
	m_nDCHeight = rcRect.Height()*2;

	CDC *lpDC = GetDC();
	m_dcBuffer.CreateCompatibleDC( lpDC );
	m_bmpBuffer.CreateBitmap ( m_nDCWidth, m_nDCHeight, 1, 32, NULL);
	m_dcBuffer.SelectObject( m_bmpBuffer );
}

void CPWheel::SetValue(FLOAT fPhase)
{
	m_fPhase = fPhase;
}

FLOAT CPWheel::GetValue()
{
	return m_fPhase;
}
