// PointGraph.cpp : implementation file
//

#include "stdafx.h"
#include "ECG.h"
#include "PointGraph.h"
#include "Graph.h"

#define GRAPH_CLASSNAME    _T("PointGraphCtrl")  // Window class name

BOOL CPointGraph::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, GRAPH_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class

        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = GRAPH_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}

// CPointGraph

IMPLEMENT_DYNAMIC(CPointGraph, CWnd)

CPointGraph::CPointGraph()
{
	RegisterWindowClass();
}

CPointGraph::~CPointGraph()
{
}


BEGIN_MESSAGE_MAP(CPointGraph, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

void CPointGraph::OnPaint()
{
	if ( m_nWidth <= 0 )
		return;

	CPaintDC dc(this); // device context for painting
	CDC *lpDC = &m_dcBuffer;

	CRect rcRect, rcWndRect;
	GetClientRect( &rcWndRect );
	rcRect = rcWndRect;
	rcRect.right += rcRect.Width();
	rcRect.bottom += rcRect.Height();

	int i;
	CRect rcSub = rcRect;

	COLORREF clrDelta = CGraph::ColorDelta( m_clrBack, -20, -20, 0 );
	for ( i = rcRect.top; i < rcRect.bottom; i+=2 )
	{
		CBrush myBrush( CGraph::ColorMid( m_clrBack, clrDelta, i*255/rcRect.bottom ) );
		rcSub.top = i;
		rcSub.bottom = i+20;
		lpDC->FillRect( rcSub, &myBrush );
	}
	CBrush brushItem( m_clrLine );

	for ( i = 0; i < m_nWidth; i++ )
	{
		CPoint ptItem = GetPoint( i );
		if ( ptItem.x == -1 )
			continue;
		CRect rcItem( ptItem + CPoint(-2, -2), ptItem + CPoint(2, 2) );

		lpDC->FillRect( &rcItem, &brushItem );
	}


	/*
	CPen penAvg(PS_SOLID, 10, m_clrBack);//ColorMid(m_clrLine, 0xffffff, 128));
	CPen penLine(PS_SOLID, 3, m_clrLine);
	CPen penLine0(PS_SOLID, 1, ColorMid( m_clrLine, m_clrBack, 250 ) );
	CPen penLine1(PS_SOLID, 1, ColorMid( m_clrLine, m_clrBack, 200 ) );
	CPen penLine2(PS_SOLID, 2, ColorMid( m_clrLine, m_clrBack, 150 ) );
	CPen penLine3(PS_SOLID, 2, ColorMid( m_clrLine, m_clrBack, 80 ) );
	CPen penLine4(PS_SOLID, 3, ColorMid( m_clrLine, m_clrBack, 20 ) );

	HPEN hOldPen = (HPEN)lpDC->SelectObject( penLine );
	HPEN hSelectedPen = NULL;
	HPEN hNewPen;

	int nSub = max(2, m_nWidth / 50);
	int nAverage = 0;
	int nSum = 0;

	if ( m_nType == 1 ) 
	{
		lpDC->SelectObject( penAvg );
		for ( i = 0; i < m_nWidth; i++ )
			nSum += m_arrData[i];
		nAverage = nSum / m_nWidth;
		m_nStatRMS = nSum - nAverage*m_nWidth;


		lpDC->MoveTo( CPoint(0, GetValue(nAverage)) );
		lpDC->LineTo( CPoint(m_nWidth, GetValue(nAverage)) );
	}

	for ( i = 0; i < m_arrMarkers.GetSize(); i++ )
	{
		CPen penVert(PS_SOLID, 1, m_arrMarkers[i].clrColor);
		HPEN hOldPen = (HPEN)lpDC->SelectObject( penVert );
		int nXVal = m_arrMarkers[i].nValue * (double)m_nDCWidth / m_nWidth;
		lpDC->MoveTo( CPoint(nXVal, rcRect.top) );
		lpDC->LineTo( CPoint(nXVal, rcRect.bottom) );
		lpDC->SelectObject( hOldPen );
	}
	// draw front
	for ( i = 0; i < m_nWidth; i++ )
	{
		if ( m_nType == 1 )
		{
			int nOds = (m_nWidth+m_nPos-i-1)%m_nWidth;
			hNewPen = penLine;
			if (nOds > m_nWidth - nSub)
				hNewPen = penLine0;
			else if (nOds > m_nWidth - nSub*2)
				hNewPen = penLine1;
			else if (nOds > m_nWidth - nSub*3)
				hNewPen = penLine2;
			else if (nOds > m_nWidth - nSub*4)
				hNewPen = penLine3;
			else if (nOds > m_nWidth - nSub*5)
				hNewPen = penLine4;

			if ( hNewPen != hSelectedPen )
			{
				hSelectedPen = hNewPen;
				lpDC->SelectObject( hNewPen );
			}
		}

		( i == 0 ) ?
			lpDC->MoveTo( GetPoint( i ) ) :
			lpDC->LineTo( GetPoint( i ) );
	}

	lpDC->SelectObject( hOldPen );*/
	if (lpDC != &dc)
	{
		dc.SetStretchBltMode(HALFTONE);
		dc.SetBrushOrg(0, 0);
		dc.StretchBlt(0, 0, rcWndRect.Width(), rcWndRect.Height(), lpDC, 0, 0, rcRect.Width(), rcRect.Height(), SRCCOPY );
	}
}

void CPointGraph::Setup(int nWidth, int nMin, int nMax, COLORREF clrBack, COLORREF clrLine)
{
	m_nWidth = nWidth;
	m_nMin = nMin;
	m_nMax = abs(nMax);
	m_clrLine = clrLine;
	m_clrBack = clrBack;
	m_nPos = 0;

	m_bInvert = (nMax < 0) ? TRUE : FALSE;

	CRect rcRect;
	GetClientRect( &rcRect );

	if ( nWidth == 0 )
		m_nWidth = rcRect.Width();
	m_arrData.SetSize( m_nWidth );

	m_nDCWidth = rcRect.Width()*2;
	m_nDCHeight = rcRect.Height()*2;
	for (int i=0; i<m_nWidth; i++)
		m_arrData.SetAt(i, m_nMin-1);

	CDC *lpDC = GetDC();
	m_dcBuffer.CreateCompatibleDC( lpDC );
	m_bmpBuffer.CreateBitmap ( m_nDCWidth, m_nDCHeight, 1, 32, NULL);
	m_dcBuffer.SelectObject( m_bmpBuffer );
}

void CPointGraph::AddValue(int nValue)
{
	m_arrData.SetAt( m_nPos, nValue );

	if ( ++m_nPos >= m_nWidth )
		m_nPos = 0;
}


CPoint CPointGraph::GetPoint( int n )
{
	int nVal = m_arrData[ (m_nWidth-n+m_nPos) % m_nWidth ];

	if ( nVal == m_nMin-1 )
		return CPoint(-1,-1);

	int nX = n * m_nDCWidth / m_nWidth;

	if ( nVal < m_nMin || nVal >= m_nMax )
		return CPoint(nX, m_nDCHeight/2);

	if ( m_bInvert )
		return CPoint( nX, m_nDCHeight - (nVal-m_nMin)*m_nDCHeight/(m_nMax-m_nMin) );

	return CPoint( nX, (nVal-m_nMin)*m_nDCHeight/(m_nMax-m_nMin) );
}

int CPointGraph::GetValue( int nVal )
{
	if ( nVal == m_nMin-1 )
		return -1;

	if ( nVal < m_nMin )
		return 0;
	if ( nVal > m_nMax )
		return m_nDCHeight;

	if ( m_bInvert )
		return m_nDCHeight - (nVal-m_nMin)*m_nDCHeight/(m_nMax-m_nMin);

	return (nVal-m_nMin)*m_nDCHeight/(m_nMax-m_nMin);
}


// CPointGraph message handlers


