// TabSynchro.cpp : implementation file
//

#include "stdafx.h"
#include "ECG.h"
#include "ECGDlg.h"
#include "TabSynchro.h"


// CTabSynchro dialog

IMPLEMENT_DYNAMIC(CTabSynchro, CDialog)

CTabSynchro::CTabSynchro(CWnd* pParent /*=NULL*/)
	: CTabDialog(CTabSynchro::IDD, pParent)
{
	m_hMidiOut = NULL;

}

CTabSynchro::~CTabSynchro()
{
	OnBnClickedButton2();
}

void CTabSynchro::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO1, m_ctlDevices);
	DDX_Control(pDX, IDC_CUSTOM3, m_pwEcg);
	DDX_Control(pDX, IDC_CUSTOM4, m_pwFinal);
	DDX_Control(pDX, IDC_SLIDER1, m_ctlOffset);
}


BEGIN_MESSAGE_MAP(CTabSynchro, CDialog)
	//ON_WM_PAINT()
	ON_BN_CLICKED(IDC_BUTTON1, &CTabSynchro::OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON2, &CTabSynchro::OnBnClickedButton2)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLIDER1, &CTabSynchro::OnNMCustomdrawSlider1)
END_MESSAGE_MAP()

BOOL CTabSynchro::OnInitDialog()
{
	CTabDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	m_pwEcg.Setup(0, 1, GetSysColor(15), RGB(255, 0, 0) );
	m_pwEcg.AddRegion( 0.02f, 0.3f, RGB(80, 0, 120) );
	m_pwFinal.Setup(0, 1, GetSysColor(15), RGB(0, 0, 128) );
	m_ctlOffset.SetRange(-1024, 1024, FALSE);
	m_ctlOffset.SetPos( 512 );
	m_ctlOffset.Invalidate(TRUE);
	m_fOffset = 0.0f;
	//m_whDelta.Setup(0, 1024, GetSysColor(15), RGB(255, 0, 0) );
	//m_whMidi.Setup(0, 1024, GetSysColor(15), RGB(255, 0, 0) );

	for (int i=0; i<(int)midiOutGetNumDevs(); i++)
	{
		CString strDevice;
		MIDIOUTCAPS midiCaps;
		midiOutGetDevCaps(i, &midiCaps, sizeof(midiCaps));
		strDevice.Format( _T("%02d: %s"), i+1, midiCaps.szPname);
		m_ctlDevices.AddString( strDevice );
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

// CTabSynchro message handlers

class CircularBuffer {
	CArray <int, int> m_Buffer;
	int m_nPos, m_nCount, m_nLength;
public:
	CircularBuffer()
	{
		m_nLength = 4096;
		m_Buffer.SetSize(m_nLength);
		m_nPos = 0;
		m_nCount = 0;
	}

	void Push(int nValue)
	{
		m_Buffer[m_nPos++] = nValue;
		if ( m_nPos >= m_nLength )
			m_nPos = 0;
		m_nCount = min( m_nLength, m_nCount+1 );
	}
	int Get(int nDelta)
	{
		nDelta = -nDelta;
		if ( nDelta >= m_nCount )
			return -1;
		int nOfs = m_nPos-nDelta+m_nLength*4;
		nOfs %= m_nLength;
		return m_Buffer[nOfs];
	}
};

class CHyperCorrelator 
{
	CircularBuffer m_Buffer;
	int m_arrRing[64];

	int m_nPeriodMin, m_nPeriodMax, m_nThresh;
	int m_nInhibit, m_nPassed, m_nPassed2;
	
	BOOL m_bValid;
	FLOAT m_fPhase;

	FLOAT sgn(FLOAT s)
	{
		if (s > 0)
			return 1;
		if (s < 0)
			return -1;
		return 0;
	}

public:
	int m_debug[128];

	CHyperCorrelator()
	{
		Init( 100, 500 );
	}

	FLOAT Correlate(int *a, int *b, int len)
	{
		/*
		static int nTemp = 0;

		FLOAT fMinSum = 999999;
		int nBias, nWinBias = -999;
		for (nBias = -20; nBias < 20; nBias++)
		{
			FLOAT fSum = 0;
			for (int i=0; i<len; i++)
			{
				FLOAT va = (a[i]+nBias)/1024.0f;
				FLOAT vb = (b[i])/1024.0f;
				fSum += (va-vb)*(va-vb);
			}
			if (fSum < fMinSum)
			{
				fMinSum = fSum;
				nWinBias = nBias;
			}
		}

			FLOAT fSum = 0;
			for (int i=0; i<len; i++)
			{
				int va = a[i];
				int vb = b[i];
				fSum += va-vb;
			}
			FLOAT nNumericBias = -fSum/len;

*/
			FLOAT fSingleSum2 = 0, fSingleSum = 0;
			for (int i=0; i<len; i++)
			{
				FLOAT va = a[i]/1024.0f;
				FLOAT vb = b[i]/1024.0f;
				fSingleSum2 += (va-vb)*(va-vb);
				fSingleSum += (va-vb);
			}
			fSingleSum /= (FLOAT)len;
			fSingleSum2 /= (FLOAT)len;

			//FLOAT fQ = -fSingleSum;
			FLOAT fSingleResult = fSingleSum2-fSingleSum*fSingleSum;
			//fSingleResult *= len;
			FLOAT fMinSum = fSingleResult * len;
/*
			if (nTemp++ % 32 == 0)
			{
				CString strDeb;
				int nNum = (int)(abs(nNumericBias)+0.5f)*sgn(nNumericBias);
				strDeb.Format( _T("Test=%d, Numeric=%f, Valid=%d, Res0=%f, Res1=%f\n"), 
					nWinBias, nNumericBias, (BOOL) (nNum==nWinBias),
					fMinSum, fSingleResult);

				OutputDebugString(strDeb);
			}
			
*/

		return fMinSum;
	}

	int Correlate(int *a, int na, int *b, int nb)
	{
		static int nLastCor = -1;
		int nIBest = -1;
		FLOAT fBest = 9999999;
		for (int i=0; i<na-nb; i++)
		{
			FLOAT fSuccess = Correlate(a+i, b, nb);
			if (nLastCor>-1)
				fSuccess += abs(nLastCor-i)*0.001f;
			if ( fSuccess < fBest )
			{
				nIBest = i;
				fBest = fSuccess;
			}
		}
		//m_debug[0] = 400+(int)(sqrt(fBest)*5000.0f);
		//m_debug[1] = m_debug[0];
		nLastCor = nIBest;
		return nIBest;
	}
	void Init( int nPeriodMin, int nPeriodMax )
	{
		m_nPeriodMin = nPeriodMin;
		m_nPeriodMax = nPeriodMax;
		m_nInhibit = 0;
		m_nPassed = -1;
		m_nPassed2 = -1;
		m_fPhase = 0;
		for (int i=0; i<128; i++)
			m_debug[i] = 0;
	}
	void SetThresh(int nValue)
	{
		m_nThresh = nValue;
	}
	void Push( int nValue )
	{

		static int nTemp = 0;
		// maly ringbuffer ktory budeme korelovat s dlhym
		for (int i=64-1; i>=0; i--)
			m_arrRing[i] = m_arrRing[i-1];
		m_arrRing[0] = nValue;

		m_Buffer.Push( nValue );
		if (m_nInhibit==0)
		{
			if ( nValue > m_nThresh )
			{
				nTemp++;
				m_nPassed2 = m_nPassed;
				m_nPassed = 0;
				m_nInhibit = m_nPeriodMin;
			}
		} else
			m_nInhibit--;
		
		m_bValid = FALSE;

		if ( m_nPassed > -1 ) 
			m_nPassed++;
		if ( m_nPassed2 > -1 ) 
			m_nPassed2++;
		// teraz m_Buffer[-m_nPassed] bude stale ukazovat na peak 

		if ( m_nPassed > -1 && m_nPassed2 > -1 )
		{
			int nHomoPos = m_nPassed2 - m_nPassed;
			int arrPrevBuffer[64];

			for (int i=0; i<64; i++)
			{
				// TODO: optimize ?
				arrPrevBuffer[i] = m_Buffer.Get(-nHomoPos-i+32-16);
				m_debug[i*2+1] = arrPrevBuffer[i];
				m_debug[i*2] = 0;
			}

			int nResult = Correlate( arrPrevBuffer, 64, m_arrRing, 40);

			for (int i=0; i<32; i++)
			{
				int doi=(nResult+i)*2;
				if ( doi > 0 && doi < 128)
					m_debug[doi] = m_arrRing[i];
			}

			// za vhodnych okolnosti bude vysledok okolo nuly

			int nCorDiff = nResult-16;
			FLOAT fPhase = (m_nPassed-nCorDiff) / (FLOAT ) (m_nPassed2 - m_nPassed);
			//FLOAT fPhase = (nResult-16)*0.01f;
			m_bValid = TRUE;
			m_fPhase = fPhase;
			//m_fPhase = (nResult-32+16)*0.01f;
		}
//		m_fPhase = nTemp * 0.1;
	}

	FLOAT Get()
	{
		return m_fPhase;
	}
};

CHyperCorrelator m_cor;

void CTabSynchro::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	for (int i=0; i<128; i++)
	{
		dc.MoveTo(i+100, 160);
		dc.LineTo(i+100, 160-m_cor.m_debug[i]/10);
	}
}

void CTabSynchro::ValueHandler( int nVal ) 
{
	static FLOAT fLastPhase = 0;
	static BOOL bFirst = TRUE;
	int nValue = nVal;
	int nTotalTicks = 24;
	//static int nCurrentTick = 0;
	int nSkip = 0;
	//
	// MIDI Real Time Messages
	//


	enum
	{
		TIMING_CLOCK	=0xf8,
		MEASURE_END	=0xf9,	// proposed measure end byte
		START		=0xfa,
		CONTINUE	=0xfb,
		STOP		=0xfc,
		ACTIVE_SENSE	=0xfe		
	};

	if (bFirst)
	{
		bFirst = FALSE;
	}
//	InvalidateRect(NULL);

	ECGData *lpData = &m_lpECGDlg->m_ECGData;

	DWORD dwTick = GetTickCount();
	m_cor.SetThresh( lpData->m_nThresh );
	m_cor.Push( nVal );
	m_fOffset = m_ctlOffset.GetPos()/1024.0;
	FLOAT fPhase = m_cor.Get() + m_fOffset;
	DWORD dwPassed = GetTickCount() - dwTick;
	/*
	CString strRes;
	strRes.Format( _T("passed=%d ms\n"), dwPassed );
	OutputDebugString(strRes);
	return;
*/

	m_pwEcg.SetValue( fPhase );
	m_pwEcg.Invalidate( FALSE );

	FLOAT fDist0 = fPhase-fLastPhase;
	if ( fDist0 < 0 )
		fDist0 = 1.0f - fDist0;
	
	if ( fDist0 > 0.0f && fDist0 < 0.2f )
	{
		BYTE bCur = 256*fPhase;
		BYTE bNew = ((m_nCurrentTick+1) % nTotalTicks) * 256 / nTotalTicks;
		signed char bDist = bNew - bCur;
		if (bDist > 0)
		{
			if ( m_hMidiOut )
			{
				if ( m_nCurrentTick == 0 )
					midiOutShortMsg(m_hMidiOut, (DWORD)START);
				midiOutShortMsg(m_hMidiOut, (DWORD)TIMING_CLOCK);
			}
			m_nCurrentTick++;
		}


		//FLOAT fNextVal = ((nCurrentTick+1) % nTotalTicks) / (FLOAT) nTotalTicks;
		//FLOAT fDistVal = fNextVal-fPhase;

		m_pwFinal.SetValue(bNew/256.0f);
		m_pwFinal.Invalidate( FALSE );
	}
	fLastPhase = fPhase;
}

void CTabSynchro::OnBnClickedButton1()
{
	// TODO: Add your control notification handler code here
	if ( m_hMidiOut )
		return;
	//
	int nDevice = m_ctlDevices.GetCurSel();
	if ( nDevice < 0 || nDevice > (int)midiOutGetNumDevs() )
		return;

	MMRESULT hRes = midiOutOpen(&m_hMidiOut, nDevice, NULL, (DWORD_PTR)NULL, CALLBACK_NULL);
	_ASSERT ( hRes == MMSYSERR_NOERROR );
	m_nCurrentTick = 0;
}

void CTabSynchro::OnBnClickedButton2()
{
	enum
	{
		TIMING_CLOCK	=0xf8,
		MEASURE_END	=0xf9,	// proposed measure end byte
		START		=0xfa,
		CONTINUE	=0xfb,
		STOP		=0xfc,
		ACTIVE_SENSE	=0xfe		
	};
	// TODO: Add your control notification handler code here
	if ( !m_hMidiOut )
		return;
	midiOutShortMsg(m_hMidiOut, (DWORD)STOP);

	midiOutClose(m_hMidiOut);
	m_hMidiOut = NULL;
}

void CTabSynchro::OnNMCustomdrawSlider1(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;
}
